@@ -244,22 +244,22 @@ static RD_INLINE RD_UNUSED void rtrim(char *str) {
         if (test_needs_auth()) {                                               \
                 TEST_SKIP("Mock cluster does not support SSL/SASL\n");         \
                 return RET;                                                    \
-        }                                                                      \
-        if (test_consumer_group_protocol() &&                                  \
-            strcmp(test_consumer_group_protocol(), "classic")) {               \
-                TEST_SKIP(                                                     \
-                    "Mock cluster cannot be used "                             \
-                    "with group.protocol=%s\n",                                \
-                    test_consumer_group_protocol());                           \
-                return RET;                                                    \
         }
 
-
 void test_conf_init(rd_kafka_conf_t **conf,
                     rd_kafka_topic_conf_t **topic_conf,
                     int timeout);
 
 
+typedef struct test_conf_log_interceptor_s test_conf_log_interceptor_t;
+
+test_conf_log_interceptor_t *
+test_conf_set_log_interceptor(rd_kafka_conf_t *conf,
+                              void (*log_cb)(const rd_kafka_t *rk,
+                                             int level,
+                                             const char *fac,
+                                             const char *buf),
+                              const char **debug_contexts);
 
 void test_msg_fmt(char *dest,
                   size_t dest_size,
@@ -323,7 +323,7 @@ struct test_msgver_s {
 
         const char *msgid_hdr; /**< msgid string is in header by this name,
                                 * rather than in the payload (default). */
-};                             /* test_msgver_t; */
+}; /* test_msgver_t; */
 
 /* Message */
 struct test_mv_m {
@@ -592,6 +592,8 @@ void test_verify_rkmessage0(const char *func,
 
 void test_consumer_subscribe(rd_kafka_t *rk, const char *topic);
 
+void test_consumer_subscribe_multi(rd_kafka_t *rk, int topic_count, ...);
+
 void test_consume_msgs_easy_mv0(const char *group_id,
                                 const char *topic,
                                 rd_bool_t txn,
@@ -674,6 +676,7 @@ void test_consumer_verify_assignment0(const char *func,
         test_consumer_verify_assignment0(__FUNCTION__, __LINE__, rk,           \
                                          fail_immediately, __VA_ARGS__)
 
+
 void test_consumer_assign(const char *what,
                           rd_kafka_t *rk,
                           rd_kafka_topic_partition_list_t *parts);
@@ -684,6 +687,14 @@ void test_consumer_unassign(const char *what, rd_kafka_t *rk);
 void test_consumer_incremental_unassign(const char *what,
                                         rd_kafka_t *rk,
                                         rd_kafka_topic_partition_list_t *parts);
+void test_consumer_assign_by_rebalance_protocol(
+    const char *what,
+    rd_kafka_t *rk,
+    rd_kafka_topic_partition_list_t *parts);
+void test_consumer_unassign_by_rebalance_protocol(
+    const char *what,
+    rd_kafka_t *rk,
+    rd_kafka_topic_partition_list_t *parts);
 void test_consumer_assign_partition(const char *what,
                                     rd_kafka_t *rk,
                                     const char *topic,
@@ -698,6 +709,7 @@ void test_consumer_close(rd_kafka_t *rk);
 
 void test_flush(rd_kafka_t *rk, int timeout_ms);
 
+int test_is_forbidden_conf_group_protocol_consumer(const char *name);
 void test_conf_set(rd_kafka_conf_t *conf, const char *name, const char *val);
 char *test_topic_conf_get(const rd_kafka_topic_conf_t *tconf, const char *name);
 int test_conf_match(rd_kafka_conf_t *conf, const char *name, const char *val);
@@ -709,6 +721,7 @@ void test_any_conf_set(rd_kafka_conf_t *conf,
                        const char *name,
                        const char *val);
 
+rd_kafka_topic_partition_list_t *test_topic_partitions(int cnt, ...);
 void test_print_partition_list(
     const rd_kafka_topic_partition_list_t *partitions);
 int test_partition_list_cmp(rd_kafka_topic_partition_list_t *al,
@@ -726,6 +739,11 @@ void test_create_topic(rd_kafka_t *use_rk,
                        const char *topicname,
                        int partition_cnt,
                        int replication_factor);
+void test_create_topic_wait_exists(rd_kafka_t *use_rk,
+                                   const char *topicname,
+                                   int partition_cnt,
+                                   int replication_factor,
+                                   int timeout);
 rd_kafka_resp_err_t test_auto_create_topic_rkt(rd_kafka_t *rk,
                                                rd_kafka_topic_t *rkt,
                                                int timeout_ms);
@@ -866,6 +884,10 @@ size_t test_mock_wait_matching_requests(
     rd_bool_t (*match)(rd_kafka_mock_request_t *request, void *opaque),
     void *opaque);
 
+void test_mock_cluster_member_assignment(rd_kafka_mock_cluster_t *mcluster,
+                                         int member_cnt,
+                                         ...);
+
 int test_error_is_not_fatal_cb(rd_kafka_t *rk,
                                rd_kafka_resp_err_t err,
                                const char *reason);
@@ -873,9 +895,7 @@ int test_error_is_not_fatal_cb(rd_kafka_t *rk,
 
 const char *test_consumer_group_protocol();
 
-int test_consumer_group_protocol_generic();
-
-int test_consumer_group_protocol_consumer();
+int test_consumer_group_protocol_classic();
 
 /**
  * @brief Calls rdkafka function (with arguments)
